@@ -42,6 +42,7 @@ from pdm.formats import FORMATS
 from pdm.formats.base import array_of_inline_tables, make_array, make_inline_table
 from pdm.models.caches import JSONFileCache
 from pdm.models.candidates import Candidate
+from pdm.models.environment import BareEnvironment
 from pdm.models.python import PythonInfo
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import get_specifier
@@ -284,7 +285,7 @@ def do_add(
     if not dry_run:
         project.add_dependencies(deps_to_update, group, dev)
         project.write_lockfile(project.lockfile, False)
-
+    _populate_requirement_names(group_deps)
     if sync:
         do_sync(
             project,
@@ -298,6 +299,14 @@ def do_add(
         )
 
 
+def _populate_requirement_names(req_mapping: dict[str, Requirement]) -> None:
+    # Update the requirement key if the name changed.
+    for key, req in list(req_mapping.items()):
+        if key and key.startswith(":empty:"):
+            req_mapping[req.identify()] = req
+            del req_mapping[key]
+
+
 def do_update(
     project: Project,
     *,
@@ -373,6 +382,8 @@ def do_update(
         dry_run=dry_run,
         hooks=hooks,
     )
+    for deps in updated_deps.values():
+        _populate_requirement_names(deps)
     if sync or dry_run:
         do_sync(
             project,
@@ -849,9 +860,8 @@ def get_latest_version(project: Project) -> str | None:
         and current_time - state["last-check"] < 60 * 60 * 24 * 7
     ):
         return cast(str, state["latest-version"])
-    with project.environment.get_finder(
-        [project.default_source], ignore_compatibility=True
-    ) as finder:
+    environment = BareEnvironment(project)
+    with environment.get_finder([project.default_source]) as finder:
         candidate = finder.find_best_match("pdm").best
     if not candidate:
         return None
