@@ -1,13 +1,13 @@
 import argparse
 
-import click
-
-from pdm import signals, termui
+from pdm import termui
 from pdm.cli import actions
 from pdm.cli.commands.base import BaseCommand
-from pdm.cli.commands.run import run_script_if_present
+from pdm.cli.hooks import HookManager
+from pdm.cli.options import skip_option
+from pdm.models.python import PythonInfo
 from pdm.project import Project
-from pdm.utils import get_user_email_from_git
+from pdm.utils import get_user_email_from_git, get_venv_like_prefix
 
 
 class Command(BaseCommand):
@@ -23,9 +23,10 @@ class Command(BaseCommand):
     def ask(self, question: str, default: str) -> str:
         if not self.interactive:
             return default
-        return click.prompt(question, default=default)
+        return termui.ask(question, default=default)
 
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
+        skip_option.add_to_parser(parser)
         parser.add_argument(
             "-n",
             "--non-interactive",
@@ -35,24 +36,48 @@ class Command(BaseCommand):
         parser.set_defaults(search_parent=False)
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
+        from pdm.cli.commands.venv.utils import get_venv_python
+
+        hooks = HookManager(project, options.skip)
         if project.pyproject_file.exists():
             project.core.ui.echo(
-                "{}".format(
-                    termui.cyan("pyproject.toml already exists, update it now.")
-                )
+                "pyproject.toml already exists, update it now.", style="cyan"
             )
         else:
-            project.core.ui.echo(
-                "{}".format(termui.cyan("Creating a pyproject.toml for PDM..."))
-            )
+            project.core.ui.echo("Creating a pyproject.toml for PDM...", style="cyan")
         self.set_interactive(not options.non_interactive)
 
         if self.interactive:
-            actions.do_use(project)
+            python = actions.do_use(project, ignore_requires_python=True, hooks=hooks)
+            if (
+                project.config["python.use_venv"]
+                and get_venv_like_prefix(python.executable) is None
+            ):
+                if termui.confirm(
+                    "Would you like to create a virtualenv with "
+                    f"[green]{python.executable}[/]?",
+                    default=True,
+                ):
+                    try:
+                        path = project._create_virtualenv()
+                        project.python = PythonInfo.from_path(get_venv_python(path))
+                    except Exception as e:  # pragma: no cover
+                        project.core.ui.echo(
+                            f"Error occurred when creating virtualenv: {e}\n"
+                            "Please fix it and create later.",
+                            style="red",
+                            err=True,
+                        )
         else:
-            actions.do_use(project, "3", True)
+            actions.do_use(project, "3", True, ignore_requires_python=True, hooks=hooks)
+        if get_venv_like_prefix(project.python.executable) is None:
+            project.core.ui.echo(
+                "You are using the PEP 582 mode, no virtualenv is created.\n"
+                "For more info, please visit https://peps.python.org/pep-0582/",
+                style="green",
+            )
         is_library = (
-            click.confirm("Is the project a library that will be uploaded to PyPI?")
+            termui.confirm("Is the project a library that will be uploaded to PyPI")
             if self.interactive
             else False
         )
@@ -81,9 +106,7 @@ class Command(BaseCommand):
             author=author,
             email=email,
             python_requires=python_requires,
+            hooks=hooks,
         )
         if self.interactive:
             actions.ask_for_import(project)
-
-
-signals.post_init.connect(run_script_if_present("post_init"), weak=False)
