@@ -1,13 +1,17 @@
 import argparse
 import sys
 
-import click
-
-from pdm import signals
+from pdm import termui
 from pdm.cli import actions
 from pdm.cli.commands.base import BaseCommand
-from pdm.cli.commands.run import run_script_if_present
-from pdm.cli.options import dry_run_option, groups_group, install_group, lockfile_option
+from pdm.cli.hooks import HookManager
+from pdm.cli.options import (
+    dry_run_option,
+    groups_group,
+    install_group,
+    lockfile_option,
+    skip_option,
+)
 from pdm.project import Project
 
 
@@ -19,6 +23,7 @@ class Command(BaseCommand):
         install_group.add_to_parser(parser)
         dry_run_option.add_to_parser(parser)
         lockfile_option.add_to_parser(parser)
+        skip_option.add_to_parser(parser)
         parser.add_argument(
             "--no-lock",
             dest="lock",
@@ -33,19 +38,25 @@ class Command(BaseCommand):
         )
 
     def handle(self, project: Project, options: argparse.Namespace) -> None:
-        if not project.meta and click._compat.isatty(sys.stdout):
+        if not project.meta and termui.is_interactive():
             actions.ask_for_import(project)
 
+        hooks = HookManager(project, options.skip)
+
         strategy = actions.check_lockfile(project, False)
         if strategy:
             if options.check:
                 project.core.ui.echo(
-                    "Please run `pdm lock` to update the lock file", err=True
+                    "Please run [green]`pdm lock`[/] to update the lock file", err=True
                 )
                 sys.exit(1)
             if options.lock:
-                project.core.ui.echo("Updating the lock file...", fg="green", err=True)
-                actions.do_lock(project, strategy=strategy, dry_run=options.dry_run)
+                project.core.ui.echo(
+                    "Updating the lock file...", style="green", err=True
+                )
+                actions.do_lock(
+                    project, strategy=strategy, dry_run=options.dry_run, hooks=hooks
+                )
 
         actions.do_sync(
             project,
@@ -55,8 +66,5 @@ class Command(BaseCommand):
             no_editable=options.no_editable,
             no_self=options.no_self,
             dry_run=options.dry_run,
+            hooks=hooks,
         )
-
-
-signals.pre_install.connect(run_script_if_present("pre_install"), weak=False)
-signals.post_install.connect(run_script_if_present("post_install"), weak=False)
